import unittest

from acai_aws.common.json_helper import JsonHelper


class JsonHelperTest(unittest.TestCase):

    def setUp(self):
        self.valid_json = '{"key": "value"}'
        self.valid_dict = {'key': 'value'}
        self.invalid_json = '{"key": "value"}{'
        self.invalid_dict = {'key', 'value'}

    def test_decode(self):
        valid_dict = JsonHelper.decode(self.valid_json)
        self.assertDictEqual(valid_dict, self.valid_dict)

    def test_decode_invalid(self):
        invalid_json = JsonHelper.decode(self.invalid_json)
        self.assertEqual(invalid_json, self.invalid_json)

    def test_decode_invalid_raise_error(self):
        try:
            JsonHelper.decode(self.invalid_json, True)
            fail = True
        except:
            fail = False
        if fail:
            self.fail('didnt raise error')

    def test_encode(self):
        valid_json = JsonHelper.encode(self.valid_dict)
        self.assertEqual(valid_json, self.valid_json)

    def test_encode_invalid(self):
        invalid = JsonHelper.encode(self.invalid_dict)
        self.assertEqual(invalid, self.invalid_dict)

    def test_encode_invalid_raise_error(self):
        try:
            invalid_json = JsonHelper.encode(self.invalid_dict, True)
            fail = True
        except:
            fail = False
        if fail:
            self.fail('didnt raise error')



import unittest

from acai_aws.common.records.exception import RecordException

from tests.mocks.kinesis import mock_event
from tests.mocks.kinesis.mock_functions import mock_kinesis_full, before_call, after_call, call_list


class KinesisRequirementsTest(unittest.TestCase):
    basic_event = mock_event.get_basic()

    def test_kinesis_decorator_with_basic_requirements(self):
        expected = {'kinesis_full': [True]}
        result = mock_kinesis_full(self.basic_event, None)
        self.assertDictEqual(result, expected)

    def test_kinesis_decorator_with_full_requirements(self):
        expected = {'kinesis_full': [True]}
        result = mock_kinesis_full(self.basic_event, None)
        self.assertDictEqual(result, expected)
        self.assertTrue(before_call.has_been_called)
        self.assertTrue(after_call.has_been_called)
        self.assertEqual('before', call_list[0])
        self.assertEqual('after', call_list[1])



import inspect
import signal

from acai_aws.apigateway.exception import ApiTimeOutException


def requirements(**kwargs):
    def decorator_func(func):

        def raise_timeout(*_):
            raise ApiTimeOutException()

        def start_timeout(timeout=None):
            if kwargs.get('timeout') is not None or timeout is not None:
                countdown = kwargs['timeout'] if kwargs.get('timeout') is not None else timeout
                signal.signal(signal.SIGALRM, raise_timeout)
                signal.alarm(countdown)

        def end_timeout():
            signal.alarm(0)

        def run_before(request, response):
            if kwargs.get('before') and callable(kwargs['before']):
                kwargs['before'](request, response, kwargs)

        def run_after(request, response):
            if kwargs.get('after') and callable(kwargs['after']):
                kwargs['after'](request, response, kwargs)

        def run_method(request, response):
            run_before(request, response)
            start_timeout(request.timeout)
            if not response.has_errors and kwargs.get('data_class') and inspect.isclass(kwargs['data_class']):
                data_class = kwargs['data_class'](request=request)
                func(data_class, response)
            elif not response.has_errors:
                func(request, response)
            end_timeout()
            if not response.has_errors:
                run_after(request, response)
            return response

        run_method.requirements = kwargs
        return run_method

    return decorator_func



import importlib.util
import unittest

from acai_aws.apigateway.openapi.handler.module import HandlerModule


class HandlerModuleTest(unittest.TestCase):

    def setUp(self):
        file_path = 'tests/mocks/apigateway/openapi/basic.py'
        import_path = 'tests.mocks.apigateway.openapi.basic'
        spec = importlib.util.spec_from_file_location(import_path, file_path)
        handler_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(handler_module)
        self.module = HandlerModule('tests/mocks/apigateway/openapi', file_path, handler_module, 'post', 'acai_aws/example')

        rc_file_path = 'tests/mocks/apigateway/openapi/response_codes.py'
        rc_import_path = 'tests.mocks.apigateway.openapi.response_codes'
        rc_spec = importlib.util.spec_from_file_location(rc_import_path, rc_file_path)
        rc_handler_module = importlib.util.module_from_spec(rc_spec)
        rc_spec.loader.exec_module(rc_handler_module)
        self.rc_module_get = HandlerModule('tests/mocks/apigateway/openapi', rc_file_path, rc_handler_module, 'get', 'acai_aws/example')
        self.rc_module_post = HandlerModule('tests/mocks/apigateway/openapi', rc_file_path, rc_handler_module, 'post', 'acai_aws/example')

    def test_file_path(self):
        assert self.module.file_path == 'tests/mocks/apigateway/openapi/basic.py'

    def test_module(self):
        assert len(dir(self.module.module)) == 12

    def test_method(self):
        assert self.module.method == 'post'

    def test_operation_id(self):
        assert self.module.operation_id == 'PostAcaiAwsExampleBasicAcaiGenerated'

    def test_route_path(self):
        assert self.module.route_path == '/acai_aws/example/basic'

    def test_deprecated(self):
        assert self.module.deprecated == False

    def test_summary(self):
        assert self.module.summary is None

    def test_tags(self):
        self.assertListEqual(['acai_aws-example'], self.module.tags)

    def test_requires_auth(self):
        assert self.module.requires_auth is None

    def test_required_headers(self):
        self.assertListEqual([], self.module.required_headers)

    def test_available_headers(self):
        self.assertListEqual([], self.module.available_headers)

    def test_required_query(self):
        self.assertListEqual([], self.module.required_query)

    def test_available_query(self):
        self.assertListEqual([], self.module.available_query)

    def test_required_path_params(self):
        self.assertListEqual([], self.module.required_path_params)

    def test_required_path_params(self):
        assert self.module.request_body_schema_name == 'post-acai-aws-example-basic-request-body'

    def test_response_body_schema_name(self):
        assert self.module.response_body_schema_name == 'post-acai-aws-example-basic-response-body'

    def test_request_body_schema(self):
        expected = {
            'title': 'UserRequest',
            'type': 'object',
            'required': ['id', 'email', 'active', 'favorites', 'notification_config'],
            'properties': {
                'id': {
                    'exclusiveMinimum': 0,
                    'title': 'Id',
                    'type': 'integer'
                },
                'email': {
                    'title': 'Email',
                    'type': 'string'
                },
                'active': {
                    'title': 'Active',
                    'type': 'boolean'
                },
                'favorites': {
                    'items': {
                        'type': 'string'
                    },
                    'title': 'Favorites',
                    'type': 'array'
                },
                'notification_config': {
                    'additionalProperties': {
                        'type': 'boolean'},
                    'title': 'Notification Config',
                    'type': 'object'
                }
            }
        }
        self.assertDictEqual(expected, self.module.request_body_schema)

    def test_response_body_schema_none(self):
        assert self.module.response_body_schema is None

    def test_response_codes_empty_when_not_set(self):
        self.assertDictEqual({}, self.module.response_codes)

    def test_response_codes_when_set(self):
        expected = {200: 'User exists', 204: 'User does not exist'}
        self.assertDictEqual(expected, self.rc_module_get.response_codes)

    def test_response_codes_with_response_schema(self):
        expected = {200: 'Resource found', 204: 'Resource not found'}
        self.assertDictEqual(expected, self.rc_module_post.response_codes)



import unittest

from acai_aws.alb.request import Request

from tests.mocks.alb import mock_request


class ALBRequestTest(unittest.TestCase):
    maxDiff = None

    def test_request_reads_alb_event_shape(self):
        request = Request(mock_request.get_basic_post())
        self.assertEqual('post', request.method)
        self.assertEqual('unit-test/v1/basic', request.path)
        self.assertEqual({'name': 'me'}, request.query_params)
        self.assertEqual({'body_key': 'body_value'}, request.body)

    def test_request_decodes_base64_body(self):
        request = Request(mock_request.get_basic_post_base64())
        self.assertEqual({'body_key': 'body_value'}, request.body)

    def test_request_target_group_arn(self):
        request = Request(mock_request.get_basic_post())
        self.assertEqual(
            'arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/acai-tg/50dc6c495c0c9188',
            request.target_group_arn,
        )

    def test_request_target_group_arn_missing_returns_empty(self):
        event = mock_request.get_basic_post()
        event['requestContext'] = {}
        request = Request(event)
        self.assertEqual('', request.target_group_arn)

    def test_request_source_ip_from_xff_header(self):
        request = Request(mock_request.get_basic_post())
        self.assertEqual('203.0.113.42', request.source_ip)

    def test_request_source_ip_missing_returns_empty(self):
        event = mock_request.get_basic_post()
        event['headers'] = {'content-type': 'application/json'}
        request = Request(event)
        self.assertEqual('', request.source_ip)

    def test_request_lowercases_headers(self):
        event = mock_request.get_basic_post()
        event['headers'] = {'X-Custom-Header': 'value', 'Content-Type': 'application/json'}
        request = Request(event)
        self.assertEqual('value', request.headers.get('x-custom-header'))


if __name__ == '__main__':
    unittest.main()



import csv

import boto3

from acai_aws.common.json_helper import JsonHelper
from acai_aws.base.event import BaseRecordsEvent
from acai_aws.s3.record import Record


class Event(BaseRecordsEvent):

    def __init__(self, event, context=None, **kwargs):
        super().__init__(event, context, **kwargs)
        self._record_class = Record

    def _post_operation_hook(self):
        self.__get_objects()

    def __get_objects(self):
        if not self._kwargs.get('get_object'):
            return
        client = boto3.client('s3', **self._kwargs.get('s3', {}))
        for record in self._records:
            s3_object_body = client.get_object(Bucket=record.bucket, Key=record.key)['Body']
            if self._kwargs.get('data_type') == 'json':
                s3_object_body = JsonHelper.decode(s3_object_body.read().decode('utf-8'), True)
            elif self._kwargs.get('data_type') == 'csv':
                csv_data = csv.DictReader(s3_object_body.read().decode('utf-8').splitlines(), delimiter=self._kwargs.get('delimiter', ','))
                s3_object_body = list(csv_data)
            record.body = s3_object_body.copy()



import unittest

from acai_aws.apigateway.endpoint import Endpoint
from acai_aws.apigateway.exception import ApiException
from acai_aws.apigateway.request import Request
from acai_aws.apigateway.resolver import Resolver

from tests.mocks.apigateway import mock_request


class ResolverTest(unittest.TestCase):
    base_path = 'unit-test/v1'
    handler_path = 'tests/mocks/apigateway/resolver/directory_handlers'
    bad_method_request = mock_request.get_basic_post_bad_method()
    basic_request = mock_request.get_basic_post()
    dynamic_request = mock_request.get_dynamic_post()
    bad_dynamic_request = mock_request.get_bad_dynamic_post()
    bad_dynamic_request_get = mock_request.get_bad_dynamic_get()
    no_dynamic_request = mock_request.get_no_dynamic_post()
    expected_path_params = {'id': '1'}

    def test_basic_resolve_works(self):
        request = Request(self.basic_request)
        resolver = Resolver(base_path=self.base_path, handlers=self.handler_path)
        endpoint = resolver.get_endpoint(request)
        self.assertTrue(isinstance(endpoint, Endpoint))

    def test_dynamic_resolve_checks_bad_route_throws_exception(self):
        request = Request(self.bad_dynamic_request)
        resolver = Resolver(base_path=self.base_path, handlers=self.handler_path)
        try:
            resolver.get_endpoint(request)
            self.assertTrue(False)
        except ApiException as api_error:
            self.assertEqual('no route found; requested dynamic route does not match endpoint route definition', api_error.message)

    def test_dynamic_resolve_checks_bad_route_definition_throws_exception(self):
        request = Request(self.bad_dynamic_request_get)
        resolver = Resolver(base_path=self.base_path, handlers=self.handler_path)
        try:
            resolver.get_endpoint(request)
            self.assertTrue(False)
        except ApiException as api_error:
            self.assertEqual('no route found; endpoint does not have proper variables in required_route', api_error.message)

    def test_dynamic_resolve_checks_no_route_throws_exception(self):
        request = Request(self.no_dynamic_request)
        resolver = Resolver(base_path=self.base_path, handlers=self.handler_path)
        try:
            resolver.get_endpoint(request)
            self.assertTrue(False)
        except ApiException as api_error:
            self.assertEqual('no route found; endpoint does have required_route configured', api_error.message)

    def test_basic_request_with_bad_method(self):
        request = Request(self.bad_method_request)
        resolver = Resolver(base_path=self.base_path, handlers=self.handler_path)
        try:
            resolver.get_endpoint(request)
            self.assertTrue(False)
        except ApiException as api_error:
            self.assertEqual('method not allowed', api_error.message)

    def test_get_endpoint_from_cache_works(self):
        request = Request(self.basic_request)
        resolver = Resolver(base_path=self.base_path, handlers=self.handler_path)
        self.assertEqual(0, resolver.cache_misses)
        resolver.get_endpoint(request)
        self.assertEqual(1, resolver.cache_misses)
        resolver.get_endpoint(request)
        self.assertEqual(1, resolver.cache_misses)



import unittest
from unittest.mock import patch

from acai_aws.apigateway.openapi.input.arguments import InputArguments


class InputArgumentsTest(unittest.TestCase):

    @patch('sys.argv', [
        'InputArguments',
        'generate-openapi',
        '--base=acai_aws/example',
        '--handlers=tests/mocks/apigateway/openapi/**/*.py',
        '--output=tests/outputs/arguments',
        '--format=json,yml',
        '--delete'
    ])
    def test_full_class(self):
        input_args = InputArguments()
        self.assertEqual('acai_aws/example', input_args.base)
        self.assertEqual('tests/mocks/apigateway/openapi/**/*.py', input_args.handlers)
        self.assertEqual('tests/outputs/arguments', input_args.output)
        self.assertListEqual(['json', 'yml'], input_args.formats)
        self.assertTrue(input_args.delete)



import os
import glob


class HandlerScanner:

    def __init__(self, handlers):
        self.__handlers = self.clean_path(handlers)

    @property
    def file_separator(self):
        return os.sep

    @property
    def handlers(self):
        return self.__handlers

    @property
    def handlers_base(self):
        return self.clean_path(self.__handlers.split('*')[0])

    def clean_path(self, dirty_path):
        return dirty_path.strip(self.file_separator)

    def get_handler_file_paths(self):
        glob_pattern = self.__get_glob_pattern()
        return glob.glob(glob_pattern, recursive=True)

    def __get_glob_pattern(self):
        if '*' in self.__handlers and '.py' in self.__handlers:
            return self.handlers
        return self.handlers + self.file_separator + '**' + self.file_separator + '*.py'



import importlib.util
import os

from acai_aws.apigateway.openapi.handler.module import HandlerModule


class HandlerImporter:

    def get_modules_from_file_paths(self, file_paths, handlers_base, base_path):
        SUPPORTED_METHODS = ['any', 'delete', 'get', 'head', 'options', 'patch', 'post', 'put']
        modules = []
        for file_path in file_paths:
            try:
                import_path = self.__get_import_path_from_file(file_path)
                module = self.import_module_from_file(file_path, import_path)
                for method in dir(module):
                    if method.lower() in SUPPORTED_METHODS:
                        modules.append(HandlerModule(handlers_base, file_path, module, method, base_path))
            except:  # noqa: E722 pragma: no cover
                pass
        return modules

    def import_module_from_file(self, file_path, import_path):
        spec = importlib.util.spec_from_file_location(import_path, file_path)
        handler_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(handler_module)
        return handler_module

    def __get_import_path_from_file(self, file_path):
        return file_path.replace(os.sep, '.').replace('.py', '')



import unittest
from moto import mock_aws
import boto3
import jsonpickle

from acai_aws.s3.event import Event
from acai_aws.s3.record import Record
from acai_aws.common.records.exception import RecordException

from tests.mocks.s3 import mock_event
from tests.mocks.s3.mock_data_class import MockS3DataClass


class S3EventTest(unittest.TestCase):
    basic_event = mock_event.get_basic()
    csv_event = mock_event.get_basic_csv()
    mock_aws = mock_aws()
    schema_path = 'tests/mocks/s3/openapi.yml'
    starting_csv_string = ['Name,Job,Age,Income', 'Alice,Programmer,23,110000', 'Bob,Executive,34,90000', 'Carl,Sales,45,50000']
    expected_json_data = {
        'lang': 'en-us',
        'sms': False,
        'email': True,
        'push': True
    }
    expected_csv_data = [
        {'Name': 'Alice', 'Job': 'Programmer', 'Age': '23', 'Income': '110000'},
        {'Name': 'Bob', 'Job': 'Executive', 'Age': '34', 'Income': '90000'},
        {'Name': 'Carl', 'Job': 'Sales', 'Age': '45', 'Income': '50000'}
    ]

    def setUp(self):
        self.mock_aws.start()
        self.bucket_name = self.basic_event['Records'][0]['s3']['bucket']['name']
        self.s3_json_key = self.basic_event['Records'][0]['s3']['object']['key']
        self.s3_csv_key = self.csv_event['Records'][0]['s3']['object']['key']
        s3 = boto3.resource('s3')
        bucket = s3.Bucket(self.bucket_name)
        bucket.create(CreateBucketConfiguration={'LocationConstraint': 'us-east-2'})
        json_data = jsonpickle.dumps(self.expected_json_data, unpicklable=False, use_decimal=True)
        json_data = bytes(json_data.encode('UTF-8'))
        s3_json_object = s3.Object(self.bucket_name, self.s3_json_key)
        s3_json_object.put(Body=json_data)
        csv_data = bytes('\n'.join(self.starting_csv_string).encode('UTF-8'))
        s3_csv_object = s3.Object(self.bucket_name, self.s3_csv_key)
        s3_csv_object.put(Body=csv_data)

    def tearDown(self):
        self.mock_aws.stop()

    def test_event_accepts_event(self):
        event = Event(self.basic_event)
        self.assertEqual(event.context, None)
        self.assertEqual(event.data_class, None)
        self.assertDictEqual(event.event, self.basic_event)
        self.assertEqual(len(event.records), len(self.basic_event['Records']))

    def test_event_returns_record_event(self):
        event = Event(self.basic_event)
        self.assertTrue(isinstance(event.records[0], Record))

    def test_event_returns_data_class(self):
        event = Event(self.basic_event)
        event.data_class = MockS3DataClass
        self.assertTrue(isinstance(event.records[0], MockS3DataClass))
        self.assertTrue(isinstance(event.records[0].record, Record))

    def test_event_can_get_json_object(self):
        event = Event(self.basic_event, get_object=True, data_type='json')
        self.assertDictEqual(event.records[0].body, self.expected_json_data)

    def test_event_can_get_csv_object(self):
        event = Event(self.csv_event, get_object=True, data_type='csv')
        self.assertCountEqual(event.records[0].body, self.expected_csv_data)

    def test_event_validate_record_body_with_schema_file(self):
        event = Event(
            self.basic_event,
            get_object=True,
            data_type='json',
            required_body='v1-s3-body',
            schema=self.schema_path
        )
        self.assertDictEqual(event.records[0].body, self.expected_json_data)

    def test_event_validate_record_body_with_schema_dict(self):
        schema = {
            '$id': 'https://example.com/person.schema.json',
            '$schema': 'https://json-schema.org/draft/2020-12/schema',
            'title': 'Person',
            'type': 'object',
            "properties": {
                'lang': {
                    'type': 'string'
                },
                'sms': {
                    'type': 'boolean'
                },
                'email': {
                    'type': 'boolean'
                },
                'push': {
                    'type': 'boolean'
                }
            }
        }
        event = Event(
            self.basic_event,
            get_object=True,
            data_type='json',
            required_body='v1-s3-body',
            schema=schema
        )
        self.assertDictEqual(event.records[0].body, self.expected_json_data)

    def test_event_errors_record_body_with_schema_dict(self):
        schema = {
            '$id': 'https://example.com/person.schema.json',
            '$schema': 'https://json-schema.org/draft/2020-12/schema',
            'title': 'Person',
            'type': 'object',
            "properties": {
                'lang': {
                    'type': 'integer'
                },
                'sms': {
                    'type': 'boolean'
                },
                'email': {
                    'type': 'boolean'
                },
                'push': {
                    'type': 'boolean'
                }
            }
        }
        event = Event(
            self.basic_event,
            get_object=True,
            data_type='json',
            required_body='v1-s3-body',
            schema=schema,
            raise_body_error=True
        )
        try:
            print(event.records)
            self.assertTrue(False)
        except RecordException as record_error:
            self.assertTrue(isinstance(record_error, RecordException))

    def test_event_validate_filters_out_record_body_with_schema_file(self):
        event = Event(
            self.basic_event,
            get_object=True,
            data_type='json',
            schema=self.schema_path,
            required_body='v1-s3-body-wrong'
        )
        self.assertEqual(len(event.records), 0)

    def test_event_validate_errors_out_record_body_with_schema_file(self):
        event = Event(
            self.basic_event,
            get_object=True,
            data_type='json',
            schema=self.schema_path,
            required_body='v1-s3-body-wrong',
            raise_body_error=True
        )
        try:
            print(event.records)
            self.assertTrue(False)
        except RecordException as record_error:
            self.assertTrue(isinstance(record_error, RecordException))

    def test_event_raw_records(self):
        event = Event(self.basic_event)
        self.assertCountEqual(event.raw_records, self.basic_event['Records'])

    def test_event_print(self):
        event = Event(self.basic_event)
        try:
            print(event)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import unittest

from pydantic import BaseModel

from acai_aws.common.schema import Schema
from tests.mocks.common.mock_pydantic_class import Request


class SchemaTest(unittest.TestCase):
    maxDiff = None
    schema_path = 'tests/mocks/common/openapi.yml'
    schema_dict = {
        'type': 'object',
        'required': ['id', 'body'],
        'additionalProperties': False,
        'properties': {
            'id': {
                'type': 'string'
            },
            'body': {
                'type': 'object'
            },
            'dict': {
                'type': 'boolean'
            }
        }
    }
    expected_dict_from_path = {
        'type': 'object',
        'required': ['id', 'body'],
        'additionalProperties': False,
        'properties': {
            'id': {
                'type': 'string'
            },
            'body': {
                'type': 'object'
            },
            'file': {
                'type': 'boolean'
            }
        }
    }
    expected_dict_from_dict = {
        'type': 'object',
        'required': ['id', 'body'],
        'additionalProperties': False,
        'properties': {
            'id': {
                'type': 'string'
            },
            'body': {
                'type': 'object'
            },
            'dict': {
                'type': 'boolean'
            }
        }
    }
    expected_combined_dict = {
        'type': 'object',
        'properties': {
            'test_id': {
                'type': 'string'
            },
            'object_key': {
                'type': 'object',
                'properties': {
                    'string_key': {
                        'type': 'string'
                    }
                }
            },
            'array_number': {
                'type': 'array',
                'items': {
                    'type': 'number'
                }
            },
            'array_objects': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'array_string_key': {
                            'type': 'string'
                        },
                        'array_number_key': {
                            'type': 'number'
                        }
                    }
                }
            },
            'fail_id': {
                'type': 'string'
            }
        },
        'required': ['test_id', 'object_key', 'array_number', 'array_objects'],
        'additionalProperties': False
    }
    expected_route_spec = {
        'tags': [
            'unit-test'
        ],
        'operationId': 'GetSchema',
        'deprecated': False,
        'parameters': [
            {
                'in': 'query',
                'name': 'test_id',
                'required': False,
                'schema': {
                    'type': 'string'
                }
            },
            {
                'in': 'query',
                'name': 'unit_id',
                'required': True,
                'schema': {
                    'type': 'string'
                }
            }
        ],
        'responses': {
            '200': {
                'description': 'valid',
                'content': {
                    'application/json': {
                        'schema': {
                            'type': 'object',
                            'properties': {
                                'page_number': {
                                    'type': 'number'
                                },
                                'data': {
                                    'type': 'object',
                                    'required': [
                                        'id'
                                    ],
                                    'properties': {
                                        'id': {
                                            'type': 'string'
                                        }
                                    }
                                }
                            },
                            'required': [
                                'page_number',
                                'data'
                            ]
                        }
                    }
                }
            }
        }
    }

    def test_get_openapi_spec(self):
        schema_factory = Schema(schema=self.schema_path)
        spec = schema_factory.get_openapi_spec()
        self.assertTrue(isinstance(spec, dict))

    def test_get_body_spec_from_file(self):
        schema = Schema(schema=self.schema_path)
        spec = schema.get_body_spec('v1-schema-factory-test')
        self.assertDictEqual(self.expected_dict_from_path, spec)

    def test_get_combined_body_spec_from_file(self):
        schema = Schema(schema=self.schema_path)
        spec = schema.get_body_spec('v1-test-request')
        self.assertDictEqual(self.expected_combined_dict, spec)

    def test_get_body_spec_from_dict(self):
        schema = Schema(schema=self.schema_dict)
        spec = schema.get_body_spec()
        self.assertDictEqual(self.expected_dict_from_dict, spec)

    def test_get_body_spec_from_pydantic_model(self):
        schema = Schema(schema=self.schema_path)
        spec = schema.get_body_spec(Request)
        self.assertTrue(issubclass(spec, BaseModel))

    def test_get_spec_from_route(self):
        schema = Schema(schema=self.schema_path)
        spec = schema.get_route_spec('/unit-test/v1/schema', 'get')
        self.assertDictEqual(self.expected_route_spec, spec)



import json

from acai_aws.base.event import BaseRecordsEvent
from acai_aws.alb.record import Record


class Event(BaseRecordsEvent):

    def __init__(self, event, context=None, **kwargs):
        super().__init__(event, context, **kwargs)
        self._record_class = Record
        self.__validation_errors = []

    @property
    def raw_records(self):
        return [self._event]

    @property
    def validation_errors(self):
        return self.__validation_errors

    def _dispatch_failures(self, failures):
        flattened = []
        for _record, errors in failures:
            flattened.extend(errors)
        self.__validation_errors = flattened

    def build_short_circuit_response(self):
        if not self.__validation_errors:
            return None
        return {
            'statusCode': 400,
            'headers': {'Content-Type': 'application/json'},
            'body': json.dumps({'errors': self.__validation_errors}),
            'isBase64Encoded': False,
        }



from acai_aws.apigateway.resolver.modes.base import BaseModeResolver
from acai_aws.apigateway.exception import ApiException


class PatternModeResolver(BaseModeResolver):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.__handler_pattern = kwargs['handlers']

    def _get_file_and_import_path(self, request_path):
        split_path = self.get_request_path_as_list(request_path)
        route_path = self.__get_relative_path(split_path)
        file_path = self.__handler_pattern.split(f'{self.importer.file_separator}*')[0] + self.importer.file_separator + route_path
        import_path = self.get_import_path(file_path)
        return file_path, import_path

    def __get_relative_path(self, split_path):
        file_tree = self.importer.get_handlers_file_tree()
        file_pattern = self.__get_file_pattern()
        self.__get_import_path_file_tree(split_path, 0, file_tree, file_pattern)
        return f'{self.importer.file_separator}'.join(self.import_path)

    def __get_file_pattern(self):
        split_pattern = self.__handler_pattern.split(self.importer.file_separator)
        file_pattern = split_pattern[-1]
        return file_pattern

    def __get_import_path_file_tree(self, split_path, split_index, file_tree, file_pattern):
        if split_index < len(split_path):
            import_part = None
            route_part = split_path[split_index].replace('-', '_')
            possible_directory = f'{route_part}'
            possible_file = file_pattern.replace('*', route_part)
            if possible_directory in file_tree:
                import_part = possible_directory
                split_index = split_index - 1 if split_index + 1 == len(split_path) else split_index
            elif possible_file in file_tree:
                import_part = possible_file
            elif file_tree.get('__dynamic_files') and file_tree['__dynamic_files']:
                import_part = list(file_tree['__dynamic_files'])[0]
                self.has_dynamic_route = True
                self.dynamic_parts[split_index] = split_path[split_index]
            if import_part is not None:
                self.append_import_path(import_part)
                file_leaf = self.determine_which_file_leaf(file_tree, import_part)
                index_file = file_pattern.replace('*', import_part)
                if '.py' not in import_part and split_index+1 == len(split_path) and index_file in file_leaf:
                    self.append_import_path(index_file)
                self.__get_import_path_file_tree(split_path, split_index+1, file_leaf, file_pattern)
            else:
                raise ApiException(code=404, message='route not found')



from acai_aws.common import logger
from acai_aws.common.records.exception import RecordException
from acai_aws.base.no_data import NoDataClass
from acai_aws.base.placeholder import PlaceHolderRecord
from acai_aws.common.validator import Validator


class FailureMode:
    SILENT_IGNORE = 'silent_ignore'
    LOG_WARN = 'log_warn'
    RAISE_ERROR = 'raise_error'
    RETURN_FAILURE = 'return_failure'


_VALID_FAILURE_MODES = {
    FailureMode.SILENT_IGNORE,
    FailureMode.LOG_WARN,
    FailureMode.RAISE_ERROR,
    FailureMode.RETURN_FAILURE,
}


class BaseRecordsEvent:

    def __init__(self, event, context=None, **kwargs):
        self._event = event
        self._context = context
        self._kwargs = kwargs
        self._records = []
        self._invalid_records = []
        self._record_class = PlaceHolderRecord
        self.__data_class = NoDataClass
        self.__validator = Validator(**kwargs)
        self.__validated = False

    @property
    def event(self):
        return self._event

    @property
    def context(self):
        return self._context

    @property
    def raw_records(self):
        return self._event.get('Records', [])

    @property
    def data_class(self):
        if issubclass(self.__data_class, NoDataClass):
            return None
        return self.__data_class

    @data_class.setter
    def data_class(self, data_class):
        self.__data_class = data_class

    @property
    def data_classes(self):
        return [self.data_class(record=record) for record in self._records]

    @property
    def records(self):
        if not self.__validated:
            self.validate()
        return self.data_classes if self.data_class is not None else self._records

    @property
    def invalid_records(self):
        return self._invalid_records

    @property
    def batch_failure_response_key(self):
        return 'batchItemFailures'

    def batch_item_failures(self):
        return [
            record.batch_item_identifier
            for record, _errors in self._invalid_records
            if getattr(record, 'batch_item_identifier', None) is not None
        ]

    def build_short_circuit_response(self):
        return None

    def validate(self):
        if self.__validated:
            return
        self.__validated = True
        self._build_records()
        failures = []
        failures.extend(self._run_operation_check())
        self._post_operation_hook()
        failures.extend(self._run_body_check())
        self._dispatch_failures(failures)

    def _build_records(self):
        self._records = [self._record_class(record) for record in self.raw_records]

    def _post_operation_hook(self):
        return

    def _run_operation_check(self):
        if not self._kwargs.get('operations'):
            return []
        failures = []
        surviving = []
        for record in self._records:
            if record.operation in self._kwargs['operations']:
                surviving.append(record)
            else:
                failures.append((record, [{
                    'key_path': 'operation',
                    'message': f'record operation "{record.operation}" not in allowed {self._kwargs["operations"]}',
                }]))
        self._records = surviving
        return failures

    def _run_body_check(self):
        if not self._kwargs.get('required_body'):
            return []
        failures = []
        surviving = []
        for record in self._records:
            errors = self.__validator.validate_record_body(record.body, self._kwargs['required_body'])
            if errors:
                failures.append((record, errors))
            else:
                surviving.append(record)
        self._records = surviving
        return failures

    def _dispatch_failures(self, failures):
        if not failures:
            return
        mode = self.resolve_failure_mode()
        if mode == FailureMode.RAISE_ERROR:
            record, errors = failures[0]
            raise RecordException(
                record=record,
                message=f'record failed validation; errors: {errors}',
            )
        if mode == FailureMode.LOG_WARN:
            for record, errors in failures:
                logger.log(level='WARN', log={
                    'message': 'acai_aws: record filtered by validation',
                    'record_id': getattr(record, 'batch_item_identifier', None),
                    'errors': errors,
                })
            return
        if mode == FailureMode.RETURN_FAILURE:
            self._invalid_records = failures
            return
        # FailureMode.SILENT_IGNORE — nothing to do

    def resolve_failure_mode(self):
        mode = self._kwargs.get('failure_mode')
        if mode and mode not in _VALID_FAILURE_MODES:
            raise ValueError(
                f'acai_aws: unknown failure_mode "{mode}"; '
                f'expected one of {sorted(_VALID_FAILURE_MODES)}'
            )
        if mode:
            return mode
        if self._kwargs.get('raise_body_error') or self._kwargs.get('raise_operation_error'):
            if not getattr(BaseRecordsEvent, '_deprecation_logged', False):
                logger.log(level='WARN', log={
                    'message': (
                        'acai_aws: raise_body_error/raise_operation_error are deprecated; '
                        'use failure_mode="raise_error" instead'
                    ),
                })
                BaseRecordsEvent._deprecation_logged = True
            return FailureMode.RAISE_ERROR
        return FailureMode.SILENT_IGNORE

    def __str__(self):
        return str([str(record) for record in self.records])



from acai_aws.apigateway.openapi.handler.importer import HandlerImporter
from acai_aws.apigateway.openapi.handler.scanner import HandlerScanner
from acai_aws.apigateway.openapi.input.arguments import InputArguments
from acai_aws.apigateway.openapi.input.validator import InputValidator
from acai_aws.apigateway.openapi.generator import OpenAPIGenerator
from acai_aws.apigateway.openapi.file_writer import OpenAPIFileWriter


def generate_openapi():
    print('STARTED')
    print('generating openapi docs...')
    print('validating arguments received...')
    inputs = InputArguments()
    validator = InputValidator()
    scanner = HandlerScanner(inputs.handlers)
    importer = HandlerImporter()
    generator = OpenAPIGenerator(inputs.output)
    writer = OpenAPIFileWriter()

    validator.validate_arguments(inputs)
    print('arguments validated...')
    file_paths = scanner.get_handler_file_paths()
    print(f'scanning handlers: {inputs.handlers}...')
    modules = importer.get_modules_from_file_paths(file_paths, scanner.handlers_base, inputs.base)
    print('importing handler endpoint modules...')

    for module in modules:
        generator.add_path_and_method(module)

    if inputs.delete:
        print('deleting paths and methods not found in code base')
        generator.delete_unused_paths()

    print(f'writing openapi doc to requested directory: {inputs.output}')
    writer.write_openapi(generator.doc, inputs.output, inputs.formats)
    print('COMPLETED')


if __name__ == '__main__':  # pragma: no cover
    generate_openapi()



from acai_aws.apigateway.response import Response as ApigatewayResponse


_STATUS_DESCRIPTIONS = {
    200: 'OK',
    201: 'Created',
    202: 'Accepted',
    204: 'No Content',
    301: 'Moved Permanently',
    302: 'Found',
    304: 'Not Modified',
    400: 'Bad Request',
    401: 'Unauthorized',
    403: 'Forbidden',
    404: 'Not Found',
    405: 'Method Not Allowed',
    408: 'Request Timeout',
    409: 'Conflict',
    422: 'Unprocessable Entity',
    429: 'Too Many Requests',
    500: 'Internal Server Error',
    502: 'Bad Gateway',
    503: 'Service Unavailable',
    504: 'Gateway Timeout',
}


class Response(ApigatewayResponse):

    @property
    def status_description(self):
        return f"{self.code} {_STATUS_DESCRIPTIONS.get(self.code, 'OK')}"

    @property
    def full(self):
        return {**super().full, 'statusDescription': self.status_description}



from acai_aws.base.event import BaseRecordsEvent
from acai_aws.common.records.record import Record


class Event(BaseRecordsEvent):

    def __init__(self, event, context=None, **kwargs):
        super().__init__(event, context, **kwargs)
        self._record_class = Record
        self._records = event if isinstance(event, list) else [event]

    @property
    def raw_records(self):
        return self._records

    @property
    def records(self):
        self._records = [self._record_class(record) for record in self.raw_records]
        return self.data_classes if self.data_class is not None else self._records



import copy
import json
import unittest
from unittest.mock import patch

from pydantic import BaseModel, PositiveInt

from acai_aws.base.event import BaseRecordsEvent, FailureMode
from acai_aws.common.records.exception import RecordException
from acai_aws.common.records.requirements import requirements, _merge_batch_item_failures
from acai_aws.common.validator import Validator
from acai_aws.dynamodb.record import Record as DynamoDBRecord
from acai_aws.kinesis.record import Record as KinesisRecord
from acai_aws.msk.record import Record as MSKRecord
from acai_aws.common.records.record import Record as CommonRecord
from acai_aws.sqs.event import Event as SQSEvent

from tests.mocks.sqs import mock_event as sqs_mock_event
from tests.mocks.alb import mock_event as alb_mock_event


class NotificationPreferences(BaseModel):
    lang: str
    sms: bool
    email: bool
    push: bool


class StrictUser(BaseModel):
    id: PositiveInt
    email: str


def _sqs_event_with_bad_and_good_records():
    event = copy.deepcopy(sqs_mock_event.get_basic())
    good = event['Records'][0]
    bad = copy.deepcopy(good)
    bad['messageId'] = 'bad-message-id'
    bad['body'] = '{"lang": "en-us", "sms": "not-a-bool", "email": true, "push": true}'
    event['Records'].append(bad)
    return event


class ValidatorPydanticSupportTest(unittest.TestCase):

    def test_validate_record_body_pydantic_pass(self):
        validator = Validator()
        errors = validator.validate_record_body(
            {'lang': 'en-us', 'sms': True, 'email': True, 'push': True},
            NotificationPreferences,
        )
        self.assertEqual(errors, [])

    def test_validate_record_body_pydantic_fail_returns_key_path_and_message(self):
        validator = Validator()
        errors = validator.validate_record_body(
            {'lang': 'en-us', 'sms': 'nope', 'email': True, 'push': True},
            NotificationPreferences,
        )
        self.assertEqual(len(errors), 1)
        self.assertEqual(errors[0]['key_path'], 'sms')
        self.assertIn('message', errors[0])

    def test_validate_record_body_dict_schema_uses_key_path_field(self):
        validator = Validator()
        schema = {
            'type': 'object',
            'properties': {'amount': {'type': 'integer'}},
            'required': ['amount'],
        }
        errors = validator.validate_record_body({}, schema)
        self.assertEqual(len(errors), 1)
        self.assertIn('key_path', errors[0])
        self.assertIn('message', errors[0])


class FailureModeDispatchTest(unittest.TestCase):

    def setUp(self):
        self.event = _sqs_event_with_bad_and_good_records()

    def test_silent_ignore_is_default(self):
        sqs_event = SQSEvent(self.event, required_body=NotificationPreferences)
        self.assertEqual(sqs_event.resolve_failure_mode(), FailureMode.SILENT_IGNORE)
        self.assertEqual(len(sqs_event.records), 1)
        self.assertEqual(sqs_event.invalid_records, [])

    def test_raise_error_mode_raises_eagerly(self):
        sqs_event = SQSEvent(
            self.event,
            required_body=NotificationPreferences,
            failure_mode=FailureMode.RAISE_ERROR,
        )
        with self.assertRaises(RecordException):
            sqs_event.validate()

    def test_return_failure_mode_collects_invalid_records(self):
        sqs_event = SQSEvent(
            self.event,
            required_body=NotificationPreferences,
            failure_mode=FailureMode.RETURN_FAILURE,
        )
        sqs_event.validate()
        self.assertEqual(len(sqs_event.records), 1)
        self.assertEqual(len(sqs_event.invalid_records), 1)
        failures = sqs_event.batch_item_failures()
        self.assertEqual(failures, [{'itemIdentifier': 'bad-message-id'}])

    def test_log_warn_mode_logs_and_filters(self):
        sqs_event = SQSEvent(
            self.event,
            required_body=NotificationPreferences,
            failure_mode=FailureMode.LOG_WARN,
        )
        with patch('acai_aws.base.event.logger.log') as mock_log:
            sqs_event.validate()
        self.assertEqual(len(sqs_event.records), 1)
        self.assertEqual(sqs_event.invalid_records, [])
        mock_log.assert_called()
        call_kwargs = mock_log.call_args.kwargs
        self.assertEqual(call_kwargs['level'], 'WARN')
        self.assertIn('errors', call_kwargs['log'])

    def test_unknown_failure_mode_raises_value_error(self):
        sqs_event = SQSEvent(
            self.event,
            required_body=NotificationPreferences,
            failure_mode='bogus',
        )
        with self.assertRaises(ValueError):
            sqs_event.validate()

    def test_validate_is_idempotent(self):
        sqs_event = SQSEvent(self.event, required_body=NotificationPreferences)
        sqs_event.validate()
        first_records = list(sqs_event.records)
        sqs_event.validate()
        self.assertEqual(first_records, list(sqs_event.records))

    def test_deprecated_raise_body_error_still_raises(self):
        BaseRecordsEvent._deprecation_logged = False
        sqs_event = SQSEvent(
            self.event,
            required_body=NotificationPreferences,
            raise_body_error=True,
        )
        with self.assertRaises(RecordException):
            sqs_event.validate()

    def test_operation_check_uses_failure_mode(self):
        event = _sqs_event_with_bad_and_good_records()
        sqs_event = SQSEvent(
            event,
            operations=['created'],
            failure_mode=FailureMode.RETURN_FAILURE,
        )
        sqs_event.validate()
        self.assertEqual(len(sqs_event.records), 0)
        self.assertEqual(len(sqs_event.invalid_records), 2)


class BatchItemIdentifierTest(unittest.TestCase):

    def test_sqs_identifier_uses_message_id(self):
        event = _sqs_event_with_bad_and_good_records()
        sqs_event = SQSEvent(
            event,
            required_body=NotificationPreferences,
            failure_mode=FailureMode.RETURN_FAILURE,
        )
        sqs_event.validate()
        self.assertEqual(
            sqs_event.batch_item_failures(),
            [{'itemIdentifier': 'bad-message-id'}],
        )

    def test_dynamodb_identifier_uses_sequence_number(self):
        record = DynamoDBRecord({
            'dynamodb': {'SequenceNumber': '42', 'NewImage': {'id': {'S': 'a'}}}
        })
        self.assertEqual(record.batch_item_identifier, {'itemIdentifier': '42'})

    def test_dynamodb_identifier_returns_none_when_missing(self):
        record = DynamoDBRecord({'dynamodb': {}})
        self.assertIsNone(record.batch_item_identifier)

    def test_kinesis_identifier_uses_sequence_number(self):
        record = KinesisRecord({'kinesis': {'sequenceNumber': 'abc-99'}})
        self.assertEqual(record.batch_item_identifier, {'itemIdentifier': 'abc-99'})

    def test_kinesis_identifier_returns_none_when_missing(self):
        record = KinesisRecord({'kinesis': {}})
        self.assertIsNone(record.batch_item_identifier)

    def test_msk_identifier_composes_topic_partition_offset(self):
        record = MSKRecord({'topic': 'orders', 'partition': 3, 'offset': 10007})
        self.assertEqual(
            record.batch_item_identifier,
            {'itemIdentifier': 'orders-3-10007'},
        )

    def test_msk_identifier_returns_none_when_any_part_missing(self):
        self.assertIsNone(MSKRecord({'topic': 'orders', 'partition': 3}).batch_item_identifier)
        self.assertIsNone(MSKRecord({'topic': 'orders', 'offset': 0}).batch_item_identifier)
        self.assertIsNone(MSKRecord({'partition': 0, 'offset': 0}).batch_item_identifier)

    def test_common_record_default_identifier_is_none(self):
        self.assertIsNone(CommonRecord({'any': 'dict'}).batch_item_identifier)


class MergeBatchItemFailuresTest(unittest.TestCase):

    def _event_with_one_bad_record(self):
        event = _sqs_event_with_bad_and_good_records()
        sqs_event = SQSEvent(
            event,
            required_body=NotificationPreferences,
            failure_mode=FailureMode.RETURN_FAILURE,
        )
        sqs_event.validate()
        return sqs_event

    def test_merge_wraps_none_handler_return(self):
        sqs_event = self._event_with_one_bad_record()
        result = _merge_batch_item_failures(None, sqs_event)
        self.assertEqual(
            result,
            {'batchItemFailures': [{'itemIdentifier': 'bad-message-id'}]},
        )

    def test_merge_appends_to_existing_list(self):
        sqs_event = self._event_with_one_bad_record()
        handler_result = {'batchItemFailures': [{'itemIdentifier': 'A'}]}
        merged = _merge_batch_item_failures(handler_result, sqs_event)
        self.assertEqual(
            merged['batchItemFailures'],
            [{'itemIdentifier': 'A'}, {'itemIdentifier': 'bad-message-id'}],
        )

    def test_merge_dedupes_identical_identifiers(self):
        sqs_event = self._event_with_one_bad_record()
        handler_result = {'batchItemFailures': [{'itemIdentifier': 'bad-message-id'}]}
        merged = _merge_batch_item_failures(handler_result, sqs_event)
        self.assertEqual(
            merged['batchItemFailures'],
            [{'itemIdentifier': 'bad-message-id'}],
        )

    def test_merge_preserves_other_dict_keys(self):
        sqs_event = self._event_with_one_bad_record()
        handler_result = {'ok': True, 'batchItemFailures': []}
        merged = _merge_batch_item_failures(handler_result, sqs_event)
        self.assertTrue(merged['ok'])
        self.assertEqual(merged['batchItemFailures'], [{'itemIdentifier': 'bad-message-id'}])

    def test_merge_noop_when_no_framework_failures(self):
        event = sqs_mock_event.get_basic()
        sqs_event = SQSEvent(
            event,
            required_body=NotificationPreferences,
            failure_mode=FailureMode.RETURN_FAILURE,
        )
        sqs_event.validate()
        handler_result = {'ok': True}
        merged = _merge_batch_item_failures(handler_result, sqs_event)
        self.assertEqual(merged, {'ok': True})


class DecoratorIntegrationTest(unittest.TestCase):

    def test_decorator_merges_framework_failures_into_return(self):
        captured = {}

        @requirements(
            required_body=NotificationPreferences,
            failure_mode=FailureMode.RETURN_FAILURE,
        )
        def handler(event):
            captured['record_count'] = len(event.records)
            return {'batchItemFailures': [{'itemIdentifier': 'handler-detected'}]}

        event = _sqs_event_with_bad_and_good_records()
        result = handler(event, context=None)

        self.assertEqual(captured['record_count'], 1)
        identifiers = {item['itemIdentifier'] for item in result['batchItemFailures']}
        self.assertEqual(identifiers, {'handler-detected', 'bad-message-id'})

    def test_decorator_eager_validation_raises_before_handler(self):
        called = {'handler': False}

        @requirements(
            required_body=NotificationPreferences,
            failure_mode=FailureMode.RAISE_ERROR,
        )
        def handler(event):
            called['handler'] = True
            return {}

        event = _sqs_event_with_bad_and_good_records()
        with self.assertRaises(RecordException):
            handler(event, context=None)
        self.assertFalse(called['handler'])

    def test_decorator_silent_ignore_leaves_return_untouched(self):
        @requirements(required_body=NotificationPreferences)
        def handler(event):
            return {'processed': len(event.records)}

        event = _sqs_event_with_bad_and_good_records()
        result = handler(event, context=None)
        self.assertEqual(result, {'processed': 1})


class ValidatorRequestHasSecurityTest(unittest.TestCase):

    def test_request_has_security_returns_false_without_security_block(self):
        validator = Validator()
        request = type('Req', (), {'route': '/thing', 'method': 'get'})()
        with patch('acai_aws.common.schema.Schema.get_route_spec', return_value={'responses': {}}):
            self.assertFalse(validator.request_has_security(request))

    def test_request_has_security_returns_true_with_security_block(self):
        validator = Validator()
        request = type('Req', (), {'route': '/thing', 'method': 'get'})()
        with patch('acai_aws.common.schema.Schema.get_route_spec', return_value={'security': [{'api_key': []}]}):
            self.assertTrue(validator.request_has_security(request))


class ALBDecoratorTest(unittest.TestCase):

    def test_alb_invalid_body_returns_http_400_short_circuit(self):
        handler_was_called = {'yes': False}

        @requirements(required_body=StrictUser)
        def handler(event):
            handler_was_called['yes'] = True
            return {'statusCode': 200, 'body': '{}'}

        event = alb_mock_event.get_basic()
        result = handler(event, context=None)

        self.assertFalse(handler_was_called['yes'])
        self.assertEqual(result['statusCode'], 400)
        self.assertEqual(result['headers']['Content-Type'], 'application/json')
        body = json.loads(result['body'])
        self.assertIn('errors', body)
        self.assertTrue(len(body['errors']) > 0)
        for error in body['errors']:
            self.assertIn('key_path', error)
            self.assertIn('message', error)

    def test_alb_valid_body_runs_handler(self):
        schema = {
            'type': 'object',
            'properties': {
                'business_id': {'type': 'string'},
                'decision': {'type': 'string'},
                'completed': {'type': 'string'},
            },
        }

        @requirements(required_body=schema)
        def handler(event):
            return {'statusCode': 200, 'body': 'ok'}

        event = alb_mock_event.get_basic()
        result = handler(event, context=None)
        self.assertEqual(result['statusCode'], 200)

    def test_alb_no_required_body_runs_handler(self):
        @requirements()
        def handler(event):
            return {'statusCode': 200, 'body': 'pass-through'}

        event = alb_mock_event.get_basic()
        result = handler(event, context=None)
        self.assertEqual(result['statusCode'], 200)


if __name__ == '__main__':
    unittest.main()



import base64

from acai_aws.common.json_helper import JsonHelper
from acai_aws.base.record import BaseRecord


class Record(BaseRecord):

    @property
    def id(self):
        return self._record.get('eventID')

    @property
    def name(self):
        return self._record.get('eventName')

    @property
    def source(self):
        return self._record.get('eventSource')

    @property
    def source_arn(self):
        return self._record.get('eventSourceARN')

    @property
    def region(self):
        return self._record.get('awsRegion')

    @property
    def version(self):
        return self._record.get('eventVersion')

    @property
    def invoke_identity_arn(self):
        return self._record.get('invokeIdentityArn')

    @property
    def schema_version(self):
        return self._record.get('kinesis', {}).get('kinesisSchemaVersion')

    @property
    def partition_key(self):
        return self._record.get('kinesis', {}).get('partitionKey')

    @property
    def time_stamp(self):
        return self._record.get('kinesis', {}).get('approximateArrivalTimestamp')

    @property
    def sequence_number(self):
        return self._record.get('kinesis', {}).get('sequenceNumber')

    @property
    def batch_item_identifier(self):
        sequence_number = self.sequence_number
        return {'itemIdentifier': sequence_number} if sequence_number is not None else None

    @property
    def data(self):
        b64_decoded = base64.b64decode(self._record.get('kinesis', {}).get('data')).decode('utf-8')
        return JsonHelper.decode(b64_decoded)

    @property
    def body(self):
        return self.data

    @property
    def operation(self):
        return self.UNKNOWN

    def __str__(self):
        return str({
            'id': self.id,
            'name': self.name,
            'source': self.source,
            'source_arn': self.source_arn,
            'region': self.region,
            'version': self.version,
            'invoke_identity_arn': self.invoke_identity_arn,
            'schema_version': self.schema_version,
            'partition_key': self.partition_key,
            'time_stamp': self.time_stamp,
            'sequence_number': self.sequence_number,
            'data': self.data,
            'body': self.body,
            'operation': self.operation
        })



import json
import unittest

from acai_aws.alb.event import Event
from acai_aws.alb.record import Record

from tests.mocks.alb import mock_event
from tests.mocks.alb.mock_data_class import MockALBDataClass


class ALBEventTest(unittest.TestCase):
    basic_event = mock_event.get_basic()
    schema_path = 'tests/mocks/alb/openapi.yml'
    expected_body = {
        'business_id': 'biz-e2e-001',
        'decision': 'APPROVED',
        'completed': '2026-03-30T12:40:40.353'
    }

    def test_event_accepts_event(self):
        event = Event(self.basic_event)
        self.assertEqual(event.context, None)
        self.assertEqual(event.data_class, None)
        self.assertDictEqual(event.event, self.basic_event)
        self.assertEqual(len(event.records), 1)

    def test_event_raw_records_wraps_single_event(self):
        event = Event(self.basic_event)
        self.assertEqual(len(event.raw_records), 1)
        self.assertDictEqual(event.raw_records[0], self.basic_event)

    def test_event_returns_record_instance(self):
        event = Event(self.basic_event)
        self.assertTrue(isinstance(event.records[0], Record))

    def test_event_returns_data_class(self):
        event = Event(self.basic_event)
        event.data_class = MockALBDataClass
        self.assertTrue(isinstance(event.records[0], MockALBDataClass))
        self.assertTrue(isinstance(event.records[0].record, Record))

    def test_event_filters_by_operations(self):
        event = Event(self.basic_event, operations=['created'])
        self.assertEqual(len(event.records), 1)

    def test_event_filters_out_non_matching_operations(self):
        event = Event(self.basic_event, operations=['deleted'])
        self.assertEqual(len(event.records), 0)

    def test_event_validate_record_body_with_schema_file(self):
        event = Event(self.basic_event, schema=self.schema_path, required_body='v1-alb-body')
        self.assertDictEqual(event.records[0].body, self.expected_body)

    def test_event_validate_filters_out_record_body_with_schema_file(self):
        event = Event(self.basic_event, schema=self.schema_path, required_body='v1-alb-body-wrong')
        self.assertEqual(len(event.records), 0)

    def test_event_validate_invalid_body_builds_http_400_response(self):
        event = Event(self.basic_event, schema=self.schema_path, required_body='v1-alb-body-wrong')
        event.validate()
        response = event.build_short_circuit_response()
        self.assertIsNotNone(response)
        self.assertEqual(response['statusCode'], 400)
        self.assertEqual(response['headers']['Content-Type'], 'application/json')
        self.assertFalse(response['isBase64Encoded'])
        body = json.loads(response['body'])
        self.assertIn('errors', body)
        self.assertTrue(len(body['errors']) > 0)
        for error in body['errors']:
            self.assertIn('key_path', error)
            self.assertIn('message', error)

    def test_event_validate_record_body_with_schema_dict(self):
        schema = {
            '$id': 'https://example.com/alb.schema.json',
            '$schema': 'https://json-schema.org/draft/2020-12/schema',
            'title': 'ALB',
            'type': 'object',
            'properties': {
                'business_id': {
                    'type': 'string'
                },
                'decision': {
                    'type': 'string'
                },
                'completed': {
                    'type': 'string'
                }
            }
        }
        event = Event(self.basic_event, schema=self.schema_path, required_body=schema)
        self.assertDictEqual(event.records[0].body, self.expected_body)

    def test_event_validate_invalid_body_with_dict_schema_builds_http_400(self):
        schema = {
            '$id': 'https://example.com/alb.schema.json',
            '$schema': 'https://json-schema.org/draft/2020-12/schema',
            'title': 'ALB',
            'type': 'object',
            'properties': {
                'business_id': {
                    'type': 'integer'
                },
                'decision': {
                    'type': 'string'
                }
            }
        }
        event = Event(self.basic_event, schema=self.schema_path, required_body=schema)
        event.validate()
        response = event.build_short_circuit_response()
        self.assertIsNotNone(response)
        self.assertEqual(response['statusCode'], 400)
        body = json.loads(response['body'])
        self.assertIn('errors', body)
        self.assertTrue(len(body['errors']) > 0)

    def test_event_validate_valid_body_no_short_circuit(self):
        event = Event(self.basic_event, schema=self.schema_path, required_body='v1-alb-body')
        event.validate()
        self.assertIsNone(event.build_short_circuit_response())
        self.assertEqual(len(event.validation_errors), 0)

    def test_event_validate_no_required_body_no_short_circuit(self):
        event = Event(self.basic_event)
        event.validate()
        self.assertIsNone(event.build_short_circuit_response())

    def test_event_validate_filters_out_record_body_with_schema_dict(self):
        schema = {
            '$id': 'https://example.com/alb.schema.json',
            '$schema': 'https://json-schema.org/draft/2020-12/schema',
            'title': 'ALB',
            'type': 'object',
            'properties': {
                'business_id': {
                    'type': 'integer'
                },
                'decision': {
                    'type': 'string'
                }
            }
        }
        event = Event(self.basic_event, schema=self.schema_path, required_body=schema)
        self.assertEqual(len(event.records), 0)

    def test_event_print(self):
        event = Event(self.basic_event)
        try:
            print(event)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import io
import json
import os
from contextlib import redirect_stdout
from unittest import TestCase, mock

from acai_aws.common import logger
from acai_aws.common.logger.decorator import log

def some_log_condition(*args, **_):
    if args[0] == 1:
        return True
    return False

@log()
def mock_func_simple(arg1, arg2, **kwargs):
    return {'args': [arg1, arg2], 'kwargs': kwargs}

@log(level='INFO')
def mock_func_level(arg1, arg2, **kwargs):
    return {'args': [arg1, arg2], 'kwargs': kwargs}

@log(level='INFO', condition=some_log_condition)
def mock_func_condition(arg1, arg2, **kwargs):
    return {'args': [arg1, arg2], 'kwargs': kwargs}


class LoggerTest(TestCase):

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local'})
    def test_logger_logs_simple_local_json(self):
        logger.log(level='ERROR', log={'error': 'test-simple'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'INFO', 'LOG_FORMAT': 'INLINE'})
    def test_logger_logs_simple_local_inline(self):
        logger.log(level='INFO', log={'error': 'test-simple'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS'})
    def test_logger_logs_simple_non_local(self):
        logger.log(level='ERROR', log={'error': 'test-simple'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local'})
    def test_logger_logs_simple_local(self):
        logger.log(level='ERROR', log={'error': 'test-simple-local'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local'})
    def test_logger_logs_error_json(self):
        try:
            raise Exception('error-object')
        except Exception as error:
            logger.log(level='ERROR', log=error)

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_FORMAT': 'INLINE'})
    def test_logger_logs_error_inline(self):
        try:
            raise Exception('error-object')
        except Exception as error:
            logger.log(level='ERROR', log=error)

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_FORMAT': ' inline '})
    def test_logger_logs_error_inline_with_messy_env(self):
        logger.log(level='INFO', log={'message': 'inline-format'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_FORMAT': 'JSON'})
    def test_logger_json_output(self):
        buffer = io.StringIO()
        with redirect_stdout(buffer):
            logger.log(level='WARN', log={'json': True})
        log_output = buffer.getvalue().strip()
        self.assertNotIn('\n', log_output)
        parsed = json.loads(log_output)
        self.assertEqual('WARN', parsed['level'])
        self.assertEqual({'json': True}, parsed['log'])

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_FORMAT': 'PRETTY'})
    def test_logger_pretty_output(self):
        buffer = io.StringIO()
        with redirect_stdout(buffer):
            logger.log(level='WARN', log={'json': True})
        log_output = buffer.getvalue().strip()
        self.assertIn('\n', log_output)
        parsed = json.loads(log_output)
        self.assertEqual('WARN', parsed['level'])
        self.assertEqual({'json': True}, parsed['log'])

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_FORMAT': 'INLINE'})
    def test_logger_inline_output(self):
        buffer = io.StringIO()
        with redirect_stdout(buffer):
            logger.log(level='INFO', log={'inline': True})
        log_output = buffer.getvalue().strip()
        self.assertTrue(log_output.startswith('INFO|'))
        self.assertIn("inline': True", log_output)

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local'})
    def test_logger_logs_error_as_object(self):
        try:
            raise 'error-string'
        except Exception as error:
            logger.log(level='ERROR', log={'error': error, 'request': 'request'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'ERROR'})
    def test_logger_logs_ignore_info(self):
        logger.log(level='INFO', log={'INFO': 'ignore'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'INFO'})
    def test_logger_logs_see_info(self):
        logger.log(level='INFO', log={'INFO': 'see'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'INFO'})
    def test_log_decorator(self):
        result = mock_func_simple(1, 2, test=True)
        self.assertDictEqual({'args': [1, 2], 'kwargs': {'test': True}}, result)

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'ERROR'})
    def test_log_decorator_with_level(self):
        result = mock_func_level(1, 2, test=True)
        self.assertDictEqual({'args': [1, 2], 'kwargs': {'test': True}}, result)

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'INFO'})
    def test_log_decorator_with_condition_logs(self):
        result = mock_func_condition(1, 2, test=True)
        self.assertDictEqual({'args': [1, 2], 'kwargs': {'test': True}}, result)

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'INFO'})
    def test_log_decorator_with_condition_does_not_log(self):
        result = mock_func_condition(3, 2, test=True)
        self.assertDictEqual({'args': [3, 2], 'kwargs': {'test': True}}, result)

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'ERROR'})
    def test_logger_handles_bad_level(self):
        logger.log(level='BAD', log={'INFO': 'ignore'})

    @mock.patch.dict(os.environ, {'RUN_MODE': 'SEE-LOGS', 'LOG_STAGE_VARIABLE': 'STAGE', 'STAGE': 'local', 'LOG_LEVEL': 'ERROR', 'LOG_FORMAT': 'BAD'})
    def test_logger_handles_bad_format(self):
        logger.log(level='INFO', log={'INFO': 'ignore'})



import argparse


class InputArguments:

    def __init__(self):
        args = self.__get_command_line_args()
        self.__base = args.base
        self.__handlers = args.handlers
        self.__output = args.output or args.handlers
        self.__formats = args.format or 'yml'
        self.__delete = args.delete or False

    @property
    def base(self):
        return self.__base

    @property
    def handlers(self):
        return self.__handlers

    @property
    def output(self):
        return self.__output

    @property
    def formats(self):
        return self.__formats.split(',')

    @property
    def delete(self):
        return self.__delete

    def __get_command_line_args(self):
        parser = argparse.ArgumentParser(
            prog='Acai AWS: OpenApi Generator',
            description='Will generate an openapi yml file based on your api project'
        )
        parser.add_argument(
            'action',
            help='the action to take',
            choices=['generate-openapi']
        )
        parser.add_argument(
            '-b',
            '--base',
            help='base path of the api url; (optional) default=/',
            default='/',
            required=False
        )
        parser.add_argument(
            '-l',
            '--handlers',
            help='directory or pattern location of your handlers',
            required=True
        )
        parser.add_argument(
            '-o',
            '--output',
            help='(optional) directory location to save openapi file; defaults handlers directory location',
            required=False
        )
        parser.add_argument(
            '-f',
            '--format',
            help='(optional) comma deliminted format options (yml, json)',
            choices=['yml', 'json', 'yml,json', 'json,yml'],
            required=False
        )
        parser.add_argument(
            '-d',
            '--delete',
            help='(optional) will delete routes and methods not found in code base',
            action='store_true',
            required=False
        )
        return parser.parse_args()
